/* ===========================================================================
*
*                            PUBLIC DOMAIN NOTICE
*               National Center for Biotechnology Information
*
*  This software/database is a "United States Government Work" under the
*  terms of the United States Copyright Act.  It was written as part of
*  the author's official duties as a United States Government employee and
*  thus cannot be copyrighted.  This software/database is freely available
*  to the public for use. The National Library of Medicine and the U.S.
*  Government have not placed any restriction on its use or reproduction.
*
*  Although all reasonable efforts have been taken to ensure the accuracy
*  and reliability of the software and data, the NLM and the U.S.
*  Government do not and cannot warrant the performance or results that
*  may be obtained by using this software or data. The NLM and the U.S.
*  Government disclaim all warranties, express or implied, including
*  warranties of performance, merchantability or fitness for any particular
*  purpose.
*
*  Please cite the author in any work or product based on this material.
*
* ===========================================================================*/

#include <ncbi.h>
#include "nrdb.h"

#define SEQLINE_LEN	60

extern NRFilePtr	*files;

static void	uncompress PROTO((SeqStrPtr, UcharPtr));

Report(list)
	SeqStrPtr	list;
{
	register char	*cp;
	register long	i;
	char	*seqp;

	for (; list != NULL; list = list->chain) {
		prt_header(list);
		cp = seqp = list->seq;
		if (list->clen != 0) {
			uncompress(list, seqbuf);
			cp = seqp = (CharPtr)seqbuf;
		}
		while ((i = list->seqlen - (cp - seqp)) > 0) {
			i = MIN(i, SEQLINE_LEN);
			if (fwrite(cp, i, 1, ofp) != 1)
				fatal(1, "Report:  fwrite error:  %s", sys_errlist[errno]);
			putc('\n', ofp);
			cp += i;
		}
		if (ferror(ofp))
			fatal(1, "nrdb:  error on output file:  %s", sys_errlist[errno]);
	}
	return 0;
}


int
prt_header(list)
	SeqStrPtr	list;
{
	register SeqNamePtr	snp;
	register long	totnamelen = 0;
	int		desc_cnt = 0;

	for (snp = list->name1; snp != NULL; snp = snp->chain) {
		if (desc_max > 0 && desc_cnt >= desc_max)
			break;
		if (++desc_cnt == 1)
			putc('>', ofp);
		else
			putc(delim, ofp);
		if (snp->nrfp->dbnamelen > 0) {
			fwrite(snp->nrfp->dbname, snp->nrfp->dbnamelen, 1, ofp);
			putc(':', ofp);
		}
		fwrite(snp->name, snp->namelen, 1, ofp);
		totnamelen += (snp->namelen+1);
	}
	putc('\n', ofp);

	if (totnamelen > maxwnamelen)
		maxwnamelen = totnamelen;
	return 0;
}

static void
uncompress(ssp, seqp)
	SeqStrPtr	ssp;
	register UcharPtr	seqp;
{
	register unsigned char	*cseqp;
	register unsigned char	ch;
	unsigned char	*cseqpend;
	int		cnt;

	cseqp = (unsigned char *) ssp->seq;

	for (cseqpend = cseqp + (ssp->seqlen / 4); cseqp < cseqpend; ++cseqp, seqp += 4) {
		ch = *cseqp;
		seqp[3] = nt_btoa[ch & 0x03];
		seqp[2] = nt_btoa[(ch >>= 2) & 0x03];
		seqp[1] = nt_btoa[(ch >>= 2) & 0x03];
		seqp[0] = nt_btoa[(ch >> 2)];
	}

	cnt = (ssp->seqlen % 4);
	if (cnt > 0) {
		ch = (*cseqp >> ((3-cnt) * 2));
		while (cnt--) {
			ch >>= 2;
			seqp[cnt] = nt_btoa[ch & 0x03];
		}
	}
	seqp[ssp->seqlen % 4] = NULLB;

	return;
}
