/* ===========================================================================
*
*                            PUBLIC DOMAIN NOTICE
*               National Center for Biotechnology Information
*
*  This software/database is a "United States Government Work" under the
*  terms of the United States Copyright Act.  It was written as part of
*  the author's official duties as a United States Government employee and
*  thus cannot be copyrighted.  This software/database is freely available
*  to the public for use. The National Library of Medicine and the U.S.
*  Government have not placed any restriction on its use or reproduction.
*
*  Although all reasonable efforts have been taken to ensure the accuracy
*  and reliability of the software and data, the NLM and the U.S.
*  Government do not and cannot warrant the performance or results that
*  may be obtained by using this software or data. The NLM and the U.S.
*  Government disclaim all warranties, express or implied, including
*  warranties of performance, merchantability or fitness for any particular
*  purpose.
*
*  Please cite the author in any work or product based on this material.
*
* ===========================================================================*/

#include <ncbi.h>
#include <sys/lock.h>
#include <gish.h>
#include <gishlib.h>
#include "nrdb.h"

void
initalpha()
{
	int	i;

	for (i = 1; i < DIM(xltab); ++i)
		if (!isspace(i) && i != '\n' && i != '\r')
			xltab[i] = i;

	for (i = 'a'; i <= 'z'; ++i)
		xltab[i] = toupper(i);

	nttst['A'] = nttst['C'] = nttst['G'] = nttst['T'] = 1;
	nttst['a'] = nttst['c'] = nttst['g'] = nttst['t'] = 1;

	nt_atob['A'] = nt_atob['a'] = 0;
	nt_atob['C'] = nt_atob['c'] = 1;
	nt_atob['G'] = nt_atob['g'] = 2;
	nt_atob['T'] = nt_atob['t'] = 3;

	nt_btoa[0] = 'A';
	nt_btoa[1] = 'C';
	nt_btoa[2] = 'G';
	nt_btoa[3] = 'T';

	return;
}


VoidPtr
ckalloc(size)
	size_t	size;
{
	register char	*cp;

	if (size == 0)
		++size;
	cp = myalloc(size);
	if (cp == NULL)
		fatal(1, "Out of memory");
	return cp;
}


VoidPtr
ckalloc0(size)
	size_t	size;
{
	register char	*cp;

	if (size == 0)
		++size;
	cp = myalloc(size);
	if (cp == NULL)
		fatal(1, "Out of memory");
	Nlm_MemSet(cp, 0, size);
	return cp;
}

VoidPtr
my_dup(old, size)
	VoidPtr	old;
	size_t	size;
{
	VoidPtr	new;

	new = myalloc(size);
	Nlm_MemCpy(new, old, size);
	return new;
}

VoidPtr
myalloc(sz)
	size_t	sz;
{
	static size_t	remains;
	static void *cp;
	register void *newcp, *cp0;
	register size_t	allocsz;

	if (sz%8 != 0)
		sz += (8 - (sz%8));
	if (remains < sz) {
		allocsz = MAX(256*KBYTE, sz);
		newcp = mem_malloc(allocsz);
		if (newcp == NULL)
			fatal(1, "Out of memory (%lu Kbytes requested)", (unsigned long)allocsz/KBYTE);
		if (allocsz == sz)
			return newcp;
		remains = allocsz;
		cp = newcp;
	}
	cp0 = cp;
	cp = (char *)cp + sz;
	remains -= sz;
	return cp0;
}
